#!/usr/bin/perl -w -I.
use strict;

#  ---------------------------------------------------------------------
#   WhatsNew Engine to list recent entries
#    v1.0
#    April 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package ODL::WhatsNew::WhatsNew;


use Pure::EZXML;
use DBI;


# constructor
sub new
{
   my ($classname, $database, $dbusername, $dbpassword, $dbindex, $rootxml, $recordlimit) = @_;
   
   # cater for missing parameters
   if (! defined $classname)
   { die "This method has to be called as a constructor"; }
   if (! defined $database)
   { die "Missing database name"; }
   if (! defined $dbusername)
   { $dbusername = ''; }
   if (! defined $dbpassword)
   { $dbpassword = ''; }
   if (! defined $dbindex)
   { $dbindex = 'brindex'; }
   if (! defined $rootxml)
   { $rootxml = 'metadata'; }
   if (! defined $recordlimit)
   { $recordlimit = 3; }
   
   my $self = {
      database        => $database,
      dbusername      => $dbusername,
      dbpassword      => $dbpassword,
      
      dbindex         => $dbindex,
      
      rootxml         => $rootxml,
      
      recordlimit     => $recordlimit,
   };
   bless $self, $classname;
   
   $self->openDatabase;
   
   return $self;
}


# open the database, check for tables and create if necessary
sub openDatabase
{
   my ($self) = @_;

   $self->{'dbh'} = DBI->connect ($self->{'database'}, $self->{'dbusername'}, $self->{'dbpassword'}, { PrintError => 0 });
   
   # check for correspondence between table and configuration
   my $sth = $self->{'dbh'}->prepare ("select identifier from $self->{'dbindex'} limit 1");
   $sth->execute;
   my @row = $sth->fetchrow_array;
   $sth->finish;
   if ($#row > -1)
   {
      return;
   }
   
   # create database table for tracking identifiers
   $sth = $self->{'dbh'}->do ("create table $self->{'dbindex'} (".
                              "identifier char(100) not null, ".
                              "primary key (identifier)".
                              ")");
}


# close database
sub dispose
{
   my ($self) = @_;
   
   $self->{'dbh'}->disconnect;
}


# destructor
sub DESTROY
{
   my ($self)= @_;
   
   $self->dispose;
}


# generate list of random entries from database
sub whatsnew
{
   my ($self) = @_;

   # create sql query
   my $sqlquery = "select identifier from $self->{'dbindex'} order by rand(unix_timestamp()) limit $self->{'recordlimit'}";
   
   my @results = ();
   my $sth = $self->{'dbh'}->prepare ($sqlquery);
   $sth->execute;
   while ( my @row = $sth->fetchrow_array)
   {
      push (@results, $row[0]);
   }   
   $sth->finish;
   
   push (@results, ($#results+1));
   
   \@results;
}


# remove all entries from database in anticipation of new entries being added
sub zapRecords
{
   my ($self) = @_;
   
   $self->{'dbh'}->do ("delete from $self->{'dbindex'}");
}


# add a new entry to the database
sub addRecord
{
   my ($self, $identifier) = @_;
   
   $self->{'dbh'}->do ("insert into $self->{'dbindex'} values (\'$identifier\')");
}


1;


