#  ---------------------------------------------------------------------
#   General database implementation of ODL-Recent (v1.0) - data provider
#    v1.1
#    April 2002
#  ------------------+--------------------+-----------------------------
#   Hussein Suleman  |   hussein@vt.edu   |    www.husseinsspace.com    
#  ------------------+--------------------+-+---------------------------
#   Department of Computer Science          |        www.cs.vt.edu       
#     Digital Library Research Laboratory   |       www.dlib.vt.edu      
#  -----------------------------------------+-------------+-------------
#   Virginia Polytechnic Institute and State University   |  www.vt.edu  
#  -------------------------------------------------------+-------------


package ODL::WhatsNew::WhatsNewDP;


use Pure::EZXML;
use Pure::X2D;
use XOAI::XOAISP;
use ODL::WhatsNew::WhatsNew;

use XOAI::XOAIDP;
use vars ('@ISA');
@ISA = ("XOAI::XOAIDP");


# constructor
sub new
{
   my ($classname, $configfile) = @_;
   
   my $self = $classname->SUPER::new ();

   # get configuration from file
   my $con = new Pure::X2D ($configfile);
   $self->{'database'} = $con->param ('database', '');
   $self->{'dbusername'} = $con->param ('dbusername', '');
   $self->{'dbpassword'} = $con->param ('dbpassword', '');
   $self->{'dbindex'} = $con->param ('table', 'irdb');
   $self->{'xmlroot'} = 'metadata';
   $self->{'repositoryName'} = $con->param ('repositoryName', 'ODL Search Engine');
   $self->{'adminEmail'} = $con->param ('adminEmail', "mailto:someone\@somewhere");
   $self->{'sourcearchive'} = $con->param ('archive/url', '');
   $self->{'recordlimit'} = $con->param ('recordlimit', 3);

   # create whatsnew engine object
   $self->{'whatsnew'} = new ODL::WhatsNew::WhatsNew 
     ($self->{'database'}, $self->{'dbusername'}, 
      $self->{'dbpassword'}, $self->{'dbindex'},
      $self->{'xmlroot'}, $self->{'recordlimit'});

   $self->{'odlsearch_xmlnsprefix'} = 'http://oai.dlib.vt.edu/ODL/Search/1.0',

   bless $self, $classname;
   return $self;
}


# destructor
sub dispose
{
   my ($self) = @_;
   $self->SUPER::dispose ();
   $self->{'whatsnew'}->dispose;
}


# issue a query to the engine
sub whatsnew
{
   my ($self, $query, $start, $stop) = @_;
   return $self->{'whatsnew'}->whatsnew;
}


# component-specific identification
sub Archive_Identify
{
   my ($self) = @_;
   
   my $id = $self->SUPER::Archive_Identify ();
      
   if (! exists $id->{'description'})
   {
      $id->{'description'} = [];
   }
   my $desc = {
      'odl-description' => [[ 
         {
            'xmlns' => $self->{xoai_xmlnsprefix}.'/odl-description',
            'xsi:schemaLocation' => 
                       $self->{xoai_xmlnsprefix}.'/odl-description '.
                       $self->{xoai_xmlnsprefix}.'/odl-description.xsd'
         },
         {
            'protocol' => 'ODL-Recent',
            'version'  => '1.0',
            'mdorder'  => [ qw ( protocol version ) ]
         } 
      ]]
   };
   push (@{$id->{'description'}}, $desc);
   
   $id;
}


# retrieve records from the source archive as required
sub Archive_FormatRecord
{
   my ($self, $hashref, $metadataFormat) = @_;
   
   my $oai = new XOAI::XOAISP ($self->{sourcearchive});
   
   my $output = $oai->doXOAI ('GetRecord', 
                              { metadataPrefix => $metadataFormat,
                                identifier => $hashref->{urn} }
                             ) -> content;
   
   while (substr ($output, 0, 8) ne '<record>')
   { $output = substr ($output, 1); }
   
   while (substr ($output, -9) ne '</record>')
   { $output = substr ($output, 0, -1); }

   $output;
}


# get list of sets from archive
sub Archive_ListSets
{
   my ($self) = @_;
   
   [];
}

# ignore verb
sub Archive_ListMetadataFormats
{
   my ($self, $identifier) = @_;
   
   [];
}


# pass request through to source
sub Archive_GetRecord
{
   my ($self, $identifier) = @_;
   
   my %records = ( urn => $identifier );

   \%records;
}


# perform search operation and return list of identifiers
sub Archive_ListRecords
{
   my ($self, $set, $from, $until, $metadataPrefix, $resumptionToken) = @_;
   
   my @allrows = ();
   
   my $results = $self->whatsnew;
   
   for ( my $i = 0; $i<$#$results; $i++ )
   {
      push (@allrows, { urn => $results->[$i] });
   }
   $resumptionToken = '';
   my $responseContainer =
      "<hits ".
      "xmlns=\"$self->{'odlsearch_xmlnsprefix'}/responsecontainer\" ".
      "xsi:schemaLocation=\"$self->{'odlsearch_xmlnsprefix'}/responsecontainer ".
      "$self->{'odlsearch_xmlnsprefix'}/responsecontainer.xsd\">".
      $results->[$#$results].
      "</hits>";

   ( \@allrows, $resumptionToken, $responseContainer, $metadataPrefix );
}


# forward request to ListRecords handler
sub Archive_ListIdentifiers
{
   my ($self, $set, $from, $until, $resumptionToken) = @_;

   $self->Archive_ListRecords ($set, $from, $until, '', $resumptionToken);
}


1;

